@@ -25,6 +25,9 @@
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomevfs/gnome-vfs.h>
+#ifdef HAVE_LIBSEXY
+#include <libsexy/sexy-tooltip.h>
+#endif
 
 #include "conversation-panel.h"
 #include "gui.h"
@@ -54,6 +57,17 @@ static void     conversation_panel_clicked_word       (GtkWidget              *x
                                                        char                   *word,
                                                        GdkEventButton         *event,
                                                        ConversationPanel      *panel);
+#ifdef HAVE_LIBSEXY
+static void     conversation_panel_enter_word         (GtkWidget              *xtext,
+                                                       char                   *word,
+                                                       ConversationPanel      *panel);
+static void     conversation_panel_leave_word         (GtkWidget              *xtext,
+                                                       char                   *word,
+                                                       ConversationPanel      *panel);
+static gboolean	conversation_panel_lost_focus	      (GtkWidget	      *widget,
+						       GdkEventFocus	      *event,
+						       ConversationPanel      *panel);
+#endif
 static void     conversation_panel_set_font           (ConversationPanel      *panel);
 static void     conversation_panel_font_changed       (GConfClient            *client,
                                                        guint                   cnxn_id,
@@ -68,7 +82,12 @@ static void     timestamps_changed                    (GConfClient            *c
                                                        guint                   cnxn_id,
                                                        GConfEntry             *entry,
                                                        xtext_buffer           *buffer);
-static void     conversation_panel_print_line         (xtext_buffer           *buffer,
+static void     redundant_nickstamp_changed           (GConfClient            *client,
+                                                       guint                   cnxn_id,
+                                                       GConfEntry             *entry,
+                                                       ConversationPanel      *panel);
+static void     conversation_panel_print_line         (ConversationPanel      *panel,
+		                                       xtext_buffer           *buffer,
                                                        guchar                  *text,
                                                        int                     len,
                                                        gboolean                indent);
@@ -90,6 +109,9 @@ static void     drop_paste_filename                   (GtkAction              *a
                                                        ConversationPanel      *panel);
 static void     drop_cancel                           (GtkAction              *action,
                                                        ConversationPanel      *panel);
+static void		style_set_callback                    (GtkWidget              *widget,
+                                                       GtkStyle               *previous_style,
+                                                       void                   *data);
 static void     drag_data_received                    (GtkWidget              *widget,
                                                        GdkDragContext         *context,
                                                        gint                    x,
@@ -105,8 +127,17 @@ static void     send_file                             (gpointer                f
                                                        gpointer                user_data);
 static void     on_default_copy_activate              (GtkAction              *action,
                                                        ConversationPanel      *panel);
-static gboolean uri_is_text                           (gchar *uri);
-static gboolean check_file_size                       (gchar *uri);
+static gboolean uri_is_text                           (gchar                  *uri);
+static gboolean check_file_size                       (gchar                  *uri);
+#ifdef HAVE_LIBSEXY
+GtkWidget* get_user_vbox_infos	    	    	      (struct User            *user);
+#endif
+static void redraw_transparency                       (ConversationPanel      *panel);
+#ifdef USE_XLIB
+static GdkFilterReturn root_event_cb                  (GdkXEvent              *xev,
+                                                       GdkEventProperty       *event,
+                                                       ConversationPanel      *panel);
+#endif
 
 struct _fe_lastlog_info
 {
@@ -127,6 +158,12 @@ struct _ConversationPanelPriv
 
 	gchar          *selected_word;
 	GSList         *dropped_files;
+
+#ifdef HAVE_LIBSEXY
+	GtkWidget	*current_tooltip;
+	guint		tooltip_timeout;
+#endif
+	gboolean	redundant_nickstamps;
 };
 
 static GtkActionEntry url_actions[] = {
@@ -195,6 +232,7 @@ conversation_panel_init (ConversationPanel *panel)
 {
 	GtkWidget *frame;
 	GtkActionGroup *action_group;
+	GConfClient       *client;
 
 	panel->priv = g_new0 (ConversationPanelPriv, 1);
 	panel->priv->xtext     = gtk_xtext_new (colors, TRUE);
@@ -225,9 +263,24 @@ conversation_panel_init (ConversationPanel *panel)
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
 
+#ifdef HAVE_LIBSEXY
+	panel->priv->tooltip_timeout = 0;
+	panel->priv->current_tooltip = NULL;
+#endif
+	client = gconf_client_get_default ();
+	panel->priv->redundant_nickstamps =
+		gconf_client_get_bool (client, "/apps/xchat/main_window/redundant_nickstamps", NULL);
+	g_object_unref (client);
+
+	g_signal_connect (G_OBJECT (panel), "style_set", G_CALLBACK (style_set_callback), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "drag_data_received", G_CALLBACK (drag_data_received), panel);
 	gtk_drag_dest_set (panel->priv->xtext, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 	                   target_table, G_N_ELEMENTS (target_table), GDK_ACTION_COPY | GDK_ACTION_ASK);
+
+#ifdef USE_XLIB
+	gdk_window_set_events (gdk_get_default_root_window (), GDK_PROPERTY_CHANGE_MASK);
+	gdk_window_add_filter (gdk_get_default_root_window (), (GdkFilterFunc)root_event_cb, panel);
+#endif
 }
 
 static void
@@ -271,6 +324,11 @@ conversation_panel_realize (GtkWidget *widget)
 	conversation_panel_set_background (panel);
 
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click", G_CALLBACK (conversation_panel_clicked_word), panel);
+#ifdef HAVE_LIBSEXY
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter", G_CALLBACK (conversation_panel_enter_word), panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_leave", G_CALLBACK (conversation_panel_leave_word), panel);
+	g_signal_connect (G_OBJECT (gui.main_window), "focus-out-event", G_CALLBACK (conversation_panel_lost_focus), panel);
+#endif
 	gconf_client_notify_add (client, "/apps/xchat/main_window/use_sys_fonts",
 	                         (GConfClientNotifyFunc) conversation_panel_font_changed,       panel, NULL, NULL);
 	gconf_client_notify_add (client, "/apps/xchat/main_window/font",
@@ -281,6 +339,9 @@ conversation_panel_realize (GtkWidget *widget)
 	                         (GConfClientNotifyFunc) conversation_panel_background_changed, panel, NULL, NULL);
 	gconf_client_notify_add (client, "/apps/xchat/main_window/background_transparency",
 	                         (GConfClientNotifyFunc) conversation_panel_background_changed, panel, NULL, NULL);
+	gconf_client_notify_add (client, "/apps/xchat/main_window/redundant_nickstamps",
+	                         (GConfClientNotifyFunc) redundant_nickstamp_changed, panel, NULL, NULL);
+
 
 	g_object_unref (client);
 }
@@ -292,7 +353,7 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
         current_sess = gui.current_session;
 
 	url = url_check_word (word, len);
-	if (url == 0) {
+	if (url == 0 && current_sess) {
 		if (current_sess->type == SESS_DIALOG) {
 			if (strcmp (word, current_sess->channel) == 0)
 				return WORD_NICK;
@@ -366,7 +427,17 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/DefaultPopup");
 				/* FIXME: we should not display the copy action if no text is selected */
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+			}
+			break;
+		case WORD_DIALOG:
+			{
+				GtkWidget *menu;
+				menu = gtk_ui_manager_get_widget (gui.manager, "/DialogPopup");
+				strcpy (dialog_user.nick, gui.current_session->channel);
+				dialog_user.nick[strlen (gui.current_session->channel)] = '\0';
+				current_user = &dialog_user;
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
 		case WORD_URL:
@@ -377,7 +448,7 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				if (panel->priv->selected_word)
 					g_free (panel->priv->selected_word);
 				panel->priv->selected_word = g_strdup (word);
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
 		case WORD_NICK:
@@ -394,14 +465,14 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 					user = userlist_find (gui.current_session, word);
 					if (user) {
 						current_user = user;
-						gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+						gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 					}
 				} else if (gui.current_session->type == SESS_DIALOG) {
 					menu = gtk_ui_manager_get_widget (gui.manager, "/UserDialogPopup");
 					strcpy (dialog_user.nick, word);
 					dialog_user.nick[strlen (word)] = '\0';
 					current_user = &dialog_user;
-					gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+					gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 				}
 			}
 			break;
@@ -415,16 +486,96 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				if (panel->priv->selected_word)
 					g_free (panel->priv->selected_word);
 				panel->priv->selected_word = g_strdup (word);
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
-		case WORD_DIALOG:
-			/* FIXME: show dialog context menu */
-			break;
 		}
 	}
 }
 
+#ifdef HAVE_LIBSEXY
+static gboolean
+show_user_tooltip (gchar *nick)
+{
+	GtkWidget *tooltip;
+	ConversationPanel *panel;
+	struct User *user = NULL;
+	GdkRectangle rect;
+	GdkScreen *screen;
+	GtkWidget *vbox;
+	GdkDisplay *display;
+	
+	panel = CONVERSATION_PANEL(gui.conversation_panel);
+
+	user = userlist_find (gui.current_session, nick);
+	
+	if (user) {
+		tooltip = sexy_tooltip_new ();
+
+		display = gdk_display_get_default ();
+		gdk_display_get_pointer (display, &screen, &rect.x, &rect.y, NULL);
+		rect.y += 10;
+		rect.width = 1;
+		rect.height = 1;
+		
+		vbox = get_user_vbox_infos (user);
+		gtk_container_add (GTK_CONTAINER (tooltip), vbox);
+		sexy_tooltip_position_to_rect (SEXY_TOOLTIP (tooltip), &rect, screen);
+		gtk_widget_show (tooltip);
+
+		panel->priv->current_tooltip = tooltip;
+	}
+
+	return FALSE;
+}
+
+static void
+conversation_panel_enter_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
+{
+	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
+		case WORD_NICK:
+			{
+				panel->priv->tooltip_timeout =  
+					g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
+				break;
+			}
+	}
+}
+
+static void
+conversation_panel_remove_tooltip (ConversationPanel *panel)
+{
+	if (panel->priv->tooltip_timeout) {
+		g_source_remove (panel->priv->tooltip_timeout);
+		panel->priv->tooltip_timeout = 0;
+	}
+
+	if (panel->priv->current_tooltip) {
+		gtk_widget_destroy (panel->priv->current_tooltip);
+		panel->priv->current_tooltip = NULL;
+	}
+}
+
+static void
+conversation_panel_leave_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
+{
+	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
+		case WORD_NICK:
+			{
+				conversation_panel_remove_tooltip (panel);
+				break;
+			}
+	}
+}
+
+static gboolean
+conversation_panel_lost_focus (GtkWidget *widget, GdkEventFocus *event, ConversationPanel *panel)
+{
+	conversation_panel_remove_tooltip (panel);
+	return FALSE;
+}
+#endif
+
 static void
 conversation_panel_set_font (ConversationPanel *panel)
 {
@@ -518,6 +669,12 @@ timestamps_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, xtext
 	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
 }
 
+static void
+redundant_nickstamp_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, ConversationPanel *panel)
+{
+	panel->priv->redundant_nickstamps = gconf_value_get_bool (entry->value);
+}
+
 static void
 open_url (GtkAction *action, ConversationPanel *panel)
 {
@@ -740,8 +897,7 @@ send_file (gpointer file, gpointer user_data)
 static void
 on_default_copy_activate (GtkAction *action, ConversationPanel *panel)
 {
-	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
-	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext), clipboard);
+	conversation_panel_copy_selection (panel);
 }
 
 static gboolean
@@ -780,6 +936,21 @@ check_file_size (gchar *uri)
 	return file_size_ok;
 }
 
+static void
+style_set_callback (GtkWidget *widget, GtkStyle  *previous_style, void *data)
+{
+	GConfClient *client;
+	gint color_scheme;
+	client = gconf_client_get_default ();
+	color_scheme = gconf_client_get_int (client, "/apps/xchat/irc/color_scheme", NULL);
+	g_object_unref (client);
+	if (color_scheme == 0) {
+		load_colors (color_scheme);
+		load_palette (color_scheme);
+	}
+	conversation_panel_update_colors (CONVERSATION_PANEL (gui.conversation_panel));
+}
+
 GtkWidget *
 conversation_panel_new (void)
 {
@@ -893,7 +1064,7 @@ conversation_panel_clear (ConversationPanel *panel, struct session *sess)
 }
 
 static void
-conversation_panel_print_line (xtext_buffer *buffer, guchar *text, int len, gboolean indent)
+conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, guchar *text, int len, gboolean indent)
 {
 	int            leftlen;
 	unsigned char *tab;
@@ -919,7 +1090,19 @@ conversation_panel_print_line (xtext_buffer *buffer, guchar *text, int len, gboo
 	tab = strchr (text, '\t');
 	if (tab && tab < (text + len)) {
 		leftlen = tab - text;
-		gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, len - (leftlen + 1));
+
+		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0)
+		{
+			text = tab+1;
+			len -= leftlen;
+			gtk_xtext_append_indent (buffer, 0, 0, text, len);
+		}
+		else
+		{
+			strncpy (buffer->laststamp, text, leftlen);
+			buffer->laststamp[leftlen]=0;
+			gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
+		}
 	} else {
 		gtk_xtext_append_indent (buffer, 0, 0, text, len);
 	}
@@ -938,10 +1121,10 @@ conversation_panel_print (ConversationPanel *panel, struct session *sess, guchar
 	while (1) {
 		switch (*text) {
 		case '\0':
-			conversation_panel_print_line (buffer, last_text, len, indent);
+			conversation_panel_print_line (panel, buffer, last_text, len, indent);
 			return;
 		case '\n':
-			conversation_panel_print_line (buffer, last_text, len, indent);
+			conversation_panel_print_line (panel, buffer, last_text, len, indent);
 			text++;
 			if (*text == '\0')
 				return;
@@ -1044,3 +1227,43 @@ conversation_panel_page_down (ConversationPanel *panel)
 		value = end;
 	gtk_adjustment_set_value (adj, value);
 }
+
+void
+conversation_panel_copy_selection (ConversationPanel *panel)
+{
+	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
+	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext), clipboard);
+}
+
+void
+conversation_panel_queue_tdraw (ConversationPanel *panel)
+{
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+		gtk_widget_queue_draw (panel->priv->xtext);
+	}
+}
+
+static void
+redraw_transparency (ConversationPanel *panel)
+{
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+		gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), 1);
+	}
+}
+
+#ifdef USE_XLIB
+static GdkFilterReturn
+root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel)
+{
+	static Atom at = None;
+	XEvent *xevent = (XEvent *)xev;
+
+	if (xevent->type == PropertyNotify) {
+		if (at == None)
+			at = XInternAtom (xevent->xproperty.display, "_XROOTPMAP_ID", True);
+		if (at == xevent->xproperty.atom)
+			redraw_transparency (panel);
+	}
+	return GDK_FILTER_CONTINUE;
+}
+#endif
